/****************************************************************************
*       PWM on dsPIC33EP devices
*       Example code
*   Code Rusn PWM1 and PWM2 in Complemetary mode woth dead Time
*   PWM1 Trigers ADC interrupt and ADC reads 2 channels simultanously
*****************************************************************************/
#include "built_in.h"
//----------------------------------------------------------------------------
void oscConfig(void);
void Delay_ms (unsigned int delay);
void Mcu_Init(void);
void init_ADC(void);
void init_PWM(void);
//---------------------------------------------------------------------------
#define  NUM_CHS2SCAN 2 // Number of channels enabled for channel scan

//----------------------------------------------------------------------------
unsigned char TimerInterruptCount;
unsigned char LedCount;
unsigned int CH0_Result;
unsigned int CH1_Result;

//----------------------------------------------------------------------
 // ISR Handler
 //---------------------------------------------------------------------
 void Timer1Interrupt() iv IVT_ADDR_T1INTERRUPT ics ICS_AUTO
{
 TimerInterruptCount++;     // Increment interrupt counter
  IFS0bits.T1IF = 0;         // Clear Interrupt Flag
}
//-----------------------------------------------------------------------------
void AdcInterrupt() iv IVT_ADDR_AD1INTERRUPT ics ICS_AUTO
{
 IFS0bits.AD1IF = 0; // Clear the A/D interrupt flag bit
 CH0_Result = ADC1BUF0; // Read CH0 and CH1
 CH1_Result = ADC1BUF1;

}

//--------------------------------------------------------------------------
// main code
//--------------------------------------------------------------------------
void main()
{
   oscConfig();
   Mcu_Init();
   init_ADC();
   init_PWM();
 //-------------------------------------------------
 //main loop
 //-------------------------------------------------
 while(1)
 {


  Delay_ms(10);
  LedCount++;
  if(LedCount == 15){LATDbits.LATD5 ^= 1;LedCount = 0;}
 }
}
//----------------------------------------------------------------------------
void init_ADC(void)
{
  ANSELBbits.ANSB0 = 1;  // RB0 as analog pin for ADC
  ANSELBbits.ANSB1 = 1;  // RB1 as analog pin for ADC
  TRISBbits.TRISB0 = 1;  // RB0 as input
  TRISBbits.TRISB1 = 1;  // RB1 as input

  AD1CON1bits.FORM   = 0b00;  // Data Output Format: integer
  AD1CON1bits.SIMSAM = 1;    // Sample simutaneously
  AD1CON1bits.SSRCG  = 0;    // Sample source control bit
  AD1CON1bits.SSRC   = 0b011; // Sample Clock Source: PWM special event trigger
  AD1CON1bits.ASAM   = 1;     // Sampling begins immediately after conversion
  AD1CON1bits.AD12B  = 0; // 10-bit ADC operation

  AD1CON2bits.CSCNA = 1; // Scan Input Selections for CH0+ during Sample A bit
  AD1CON2bits.CHPS  = 0b01; //Converts CH1.2

  AD1CON3bits.ADRC = 0;	 //ADC Clock is derived from Systems Clock

 // ADC Conversion Clock Tad=Tcy*(ADCS+1)= (1/70M)*64 = 1.6us
 // ADC Conversion Time for 10-bit Tc=12*Tab = 19.2us
  AD1CON3bits.ADCS = 64;

  AD1CON4bits.ADDMAEN = 0; // DMA is not used
  AD1CON2bits.SMPI    = (NUM_CHS2SCAN-1); // 2 ADC Channel is scanned

  //AD1CSSH/AD1CSSL: A/D Input Scan Selection Register
  AD1CSSH = 0x0000;
  AD1CSSLbits.CSS0 = 1;	// Enable AN0 for channel scan
  AD1CSSLbits.CSS1 = 1;	// Enable AN1 for channel scan
  AD1CSSLbits.CSS2 = 1;	// Enable AN2 for channel scan
  AD1CSSLbits.CSS3 = 1;	// Enable AN3 for channel scan


  IFS0bits.AD1IF   = 0; // Clear the A/D interrupt flag bit
  IEC0bits.AD1IE   = 1;  // Do Not Enable A/D interrupt
  AD1CON1bits.ADON = 1;  // Turn on the A/D converter

}
//-------------------------------------------------------------------------
void init_PWM(void)
{
 PTPER = 14000;  // Fpwm = 10Khz Prescaler 1:1 Fosc = 140Mhz Tosc = 7nSec
 //PTPER = Fosc/Fpwm * prescaler
//--------------------------------------------------------------------------
//PWM1 Configuration
//---------------------------------------------------------------------
  IOCON1bits.PENH = 1;  //PWM1H is controlled by PWM module
  IOCON1bits.PENL = 1;  //PWM1L is controlled by PWM module
  IOCON1bits.PMOD = 0;  //Select Complementary Output PWM mode

  IOCON1bits.POLH = 0;  // Drive signals are active-high
  IOCON1bits.POLL = 0;  // Drive signals are active-high

  IOCON1bits.OVRENH = 0;    // Disable Override feature for shutdown PWM
  IOCON1bits.OVRENL = 0;    // Disable Override feature for shutdown PWM
  IOCON1bits.OVRDAT = 0b00; // Shut down PWM with Over ride 0 on PWMH and PWML


  PWMCON1bits.DTC = 0;     // Positive Deadtime enabled

  PWMCON1bits.IUE = 0;     // Disable Immediate duty cycle updates
  PWMCON1bits.ITB = 0;     // Select Primary Timebase mode

  FCLCON1bits.FLTMOD = 3;  // Fault Disabled

  TRGCON1bits.TRGDIV = 0;  // Trigger interrupt generated every PWM cycle
  TRGCON1bits.TRGSTRT = 0; // Trigger generated after waiting 0 PWM cycles

  PDC1 = 2800 ;
  DTR1    = 20;
  ALTDTR1 = 20;

  PHASE1 = 0;     // No phase shift
  TRIG1 = 0;      // Trigger generated at beginning of PWM active period
//---------------------------------------------------------------------
  //PWM2 Configuration
//---------------------------------------------------------------------
  IOCON2bits.PENH = 1;  //PWM2H is controlled by PWM module
  IOCON2bits.PENL = 1;  //PWM2L is controlled by PWM module
  IOCON2bits.PMOD = 0;  //Select Complementary Output PWM mode

  IOCON2bits.POLH = 0;  //Drive signals are active-high
  IOCON1bits.POLL = 0;  //Drive signals are active-high

  IOCON2bits.OVRENH = 0;  //Disable Override feature for shutdown PWM
  IOCON2bits.OVRENL = 0;  //Disable Override feature for shutdown PWM
  IOCON2bits.OVRDAT = 0b00;//Shut down PWM with Over ride 0 on PWMH and PWML


  PWMCON2bits.DTC = 0;     //Positive Deadtime enabled

  PWMCON2bits.IUE = 0;     //Disable Immediate duty cycle updates
  PWMCON2bits.ITB = 0;     //Select Primary Timebase mode

  FCLCON2bits.FLTMOD = 3;  //Fault Disabled

  TRGCON2bits.TRGDIV = 0;  //Trigger interrupt generated every PWM cycle
  TRGCON2bits.TRGSTRT = 0; //Trigger generated after waiting 0 PWM cycles

  PDC2 = 2800 ;      //Duty cycle  20%

  DTR2    = 20; //Deadtime = (100nSec)
  ALTDTR2 = 20; //ALTDeadtime = (100nSec)

  PHASE2 = -2700;// phase shift 120
  TRIG2 = 0;   // Trigger generated at beginning of PWM active period

  PTCONbits.PTEN = 1;//Enable the PWM Module

}
//----------------------------------------------------------------------------
void Mcu_Init(void)
{
  INTCON1bits.NSTDIS = 1; //No nested interrupts
//--------------------------------------------------------------
// Set All pis as digital
//-------------------------------------------------------------
 ANSELA = ANSELB = ANSELC = ANSELD = ANSELE = ANSELG = 0x0000;

//--------------------------------------------------------------
//make all pins outputs
//--------------------------------------------------------------
 TRISA = TRISB = TRISC = TRISD = TRISE = TRISG = 0x0000;

//--------------------------------------------------------------
// Switch all pins off
//--------------------------------------------------------------
 LATA = LATB = LATC = LATD = LATE = LATG = 0x0000;
 
//--------------------------------------------------------------
// Switch of all leds on PORTD
//--------------------------------------------------------------
 LATD = 0;
}
//-------------------------------------------------------------------------
void oscConfig(void)
{
/*Configure Oscillator to operate the device at 70Mhz
 Fosc= Fin*M/(N1*N2), Fcy=Fosc/2
 Fosc= 8M*70/(2*2)=140Mhz for 8M input clock */

 PLLFBD = 68;                 //M = 70
 CLKDIVbits.PLLPOST=0;        /* N1=2 */
 CLKDIVbits.PLLPRE=0;        /* N2=2 */
 OSCTUN = 0;                /* Tune FRC oscillator, if FRC is used */

/* Disable Watch Dog Timer */
 RCONbits.SWDTEN=0;
}
//------------------------------------------------------------------------
void Delay_ms (unsigned int delay)
{
 TimerInterruptCount = 0;  // Clear Interrupt counter flag
 T1CONbits.TCKPS0 = 1;
 T1CONbits.TCKPS1 = 0;     // Prescaler = 01 = 8
 PR1 = 0x22E1;             // (1ms/112nSec) = 8929 = 0x22E1
 IPC0bits.T1IP = 4;        // Set Interrupt Priority lower then ADC
 IEC0bits.T1IE = 1;        // Enable Timer1 interrupts
 T1CONbits.TON = 1;        // Enable Timer1

 while (TimerInterruptCount < delay);//Wait for Interrupt counts to equal delay
 T1CONbits.TON = 0;                  // Disable the Timer
}